/*---------------------------------------------------------------------------
 *
 * Copyright (c) 2016, congatec AG. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the BSD 2-clause license which 
 * accompanies this distribution. 
 *
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the BSD 2-clause license for more details.
 *
 * The full text of the license may be found at:        
 * http://opensource.org/licenses/BSD-2-Clause   
 *
 *---------------------------------------------------------------------------
 */

/*---------------------------------------------------------------------------
 *
 * $Header:   S:/CG/archives/CGTOOLS/CGUTIL/CGUTLCMN/cginfo.c-arc   1.2   Sep 06 2016 15:52:02   congatec  $
 *
 * Contents: Congatec board/BIOS information module.
 *
 *---------------------------------------------------------------------------
 *
 *                      R E V I S I O N   H I S T O R Y
 *
 * $Log:   S:/CG/archives/CGTOOLS/CGUTIL/CGUTLCMN/cginfo.c-arc  $
 * 
 *    Rev 1.2   Sep 06 2016 15:52:02   congatec
 * Added BSD header.
 * 
 *    Rev 1.1   May 31 2012 15:45:36   gartner
 * Updated variable definition to ease 64bit porting.
 * 
 *    Rev 1.0   Oct 30 2006 15:04:52   gartner
 * Initial revision.
 * 
 * 
 *
 *---------------------------------------------------------------------------
 */

/*---------------
 * Include files
 *---------------
 */
#include "cgutlcmn.h"
#include "cginfo.h"
#include "cgbmod.h"
#include "cgbc.h"

/*--------------
 * Externs used
 *--------------
 */

/*--------------------
 * Local definitions
 *--------------------
 */


/*-------------------------
 * Module global variables
 *-------------------------
 */

/*---------------------------------------------------------------------------
 * Name: 
 * Desc: 
 * Inp:  
 *       
 * Outp: 
 *---------------------------------------------------------------------------
 */
UINT16 CgInfoGetInfo
(
    CG_INFO_STRUCT *pCgInfoStruct,
    UINT32 flags
 )
{
    UINT16 retVal;
    unsigned char CgbcCmdWriteBuf[32] = {0};
    unsigned char CgbcCmdReadBuf[32] = {0};
    UINT32 CgbcCmdStatus = 0;

    // Prepare for infomration gathering
    if ((retVal = CgMpfaStart(FALSE)) != CG_MPFARET_OK)
    {
        return CG_RET_FAILED;
    }

    // Gather required information

    // Get system and OEM BIOS versions
    if(flags & CG_FLAG_INFO_BIOS)
    {
        CgMpfaGetSysBiosVersion( (_TCHAR*) (&(pCgInfoStruct->BaseBiosVersion)),NULL);
        CgMpfaGetOEMBiosVersion( (_TCHAR*) (&(pCgInfoStruct->OEMBiosVersion)));
    }

    // More information only available on board level
    if(g_nOperationTarget == OT_BOARD)
    {
        // Get CGOS version information
        if(flags & CG_FLAG_INFO_CGOS)
        {
            pCgInfoStruct->CgosAPIVersion = CgosLibGetVersion();
            pCgInfoStruct->CgosDrvVersion = CgosLibGetDrvVersion();
        }
        if(flags & CG_FLAG_INFO_CGBC)
        {

            CgbcCmdWriteBuf[0] = CGBC_CMD_GET_FW_REV;
            if(CgosCgbcHandleCommand(hCgos, &CgbcCmdWriteBuf[0], 1, &CgbcCmdReadBuf[0], 3, &CgbcCmdStatus))
            {
#ifndef BANDR
                sprintf((char*)(&(pCgInfoStruct->FirmwareVersion)),"CGBCP%c%c%c",CgbcCmdReadBuf[0], CgbcCmdReadBuf[1],CgbcCmdReadBuf[2]);
#else
                sprintf((char*)(&(pCgInfoStruct->FirmwareVersion)),"v%c%c%c",CgbcCmdReadBuf[0], CgbcCmdReadBuf[1],CgbcCmdReadBuf[2]);
#endif
            }
        }
        if(flags & CG_FLAG_INFO_MANU)
        {
            pCgInfoStruct->CgosBoardInfo.dwSize = sizeof(pCgInfoStruct->CgosBoardInfo);
            CgosBoardGetInfo(hCgos, &(pCgInfoStruct->CgosBoardInfo));
        }
        if(flags & CG_FLAG_INFO_BCNT)
        {
            CgosBoardGetBootCounter(hCgos, &(pCgInfoStruct->BootCount));
        }
        if(flags & CG_FLAG_INFO_RTIM)
        {
            CgosBoardGetRunningTimeMeter(hCgos, &(pCgInfoStruct->RunningTime));
        }
        // Also store state of the BIOS update protection
        pCgInfoStruct->BupState = CgMpfaCheckBUPActive();
    }

    // Perform cleanup
    if ((retVal = CgMpfaEnd()) != CG_MPFARET_OK)
    {
        return CG_RET_FAILED;
    }
    return CG_RET_OK;

}

