/*
 * Copyright (C) 2011 HBM Netherlands B.V.
 * Schutweg 15a
 * 5145NP Waalwijk
 * The Netherlands
 * http://www.hbm.com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

 /*
  * Public driver API
  */

#ifndef HBM_FPGA_DRIVER_H
#define HBM_FPGA_DRIVER_H


#define HBM_FPGA_MAGIC                          'r'
#define HBM_FPGA_IOCTL_READ_PHYSICAL_ADDRESS    _IOR(HBM_FPGA_MAGIC,  1, unsigned long)
#define HBM_FPGA_IOCTL_SET_TRACE_FLAGS          _IOW(HBM_FPGA_MAGIC,  2, int)
#define HBM_FPGA_IOCTL_READ_MEMORY_SIZE         _IOR(HBM_FPGA_MAGIC,  8, unsigned int)
#define HBM_FPGA_IOCTL_I2C_INIT                 _IOR(HBM_FPGA_MAGIC,  9, int)
#define HBM_FPGA_IOCTL_I2C_RESET                _IOR(HBM_FPGA_MAGIC, 10, int)
#define HBM_FPGA_IOCTL_I2C_MSG                  _IOR(HBM_FPGA_MAGIC, 11, i2c_msg_params)
#define HBM_FPGA_IOCTL_CHANGE_IRQ_MASK          _IOR(HBM_FPGA_MAGIC, 12, irq_params)
#define HBM_FPGA_IOCTL_GET_PCI_BUS_NUMBER       _IOR(HBM_FPGA_MAGIC, 13, unsigned char)
#define HBM_FPGA_IOCTL_GET_PCI_SUBDEVICE_ID     _IOR(HBM_FPGA_MAGIC, 14, uint32_t)

#define HBM_LCD_MAGIC                           'r'
#define HBM_LCD_IOCTL_SET_TEST_PATTERN          _IOW(HBM_LCD_MAGIC,   1, int)
#define HBM_LCD_IOCTL_SET_BACKLIGHT             _IOW(HBM_LCD_MAGIC,   2, unsigned int)

/* HBM_LCD_IOCTL_SET_TEST_PATTERN values */
#define LCD_TEST__NONE				((0x0 << 1) | 0x0)  /* No test, normal operation: Periodically copy the framebuffer into the page buffers */
#define LCD_TEST__NONE_FLASH			((0x0 << 1) | 0x1)  /* As, LCD_TEST__NONE, but invert the display every update */
#define LCD_TEST__CHECKER			((0x1 << 1) | 0x0)  /* Copy a checker board pattern in the page buffers to show on the LCD (do not show the framebuffer contents) */
#define LCD_TEST__CHECKER_FLASH			((0x1 << 1) | 0x1)  /* As, LCD_TEST__CHECKER, but invert the display every update */
#define LCD_TEST__FULL_ON			((0x2 << 1) | 0x0)  /* Copy a all 1s pattern in the page buffers to show on the LCD (do not show the framebuffer contents) */
#define LCD_TEST__FULL_ON_FLASH			((0x2 << 1) | 0x1)  /* As, LCD_TEST__FULL_ON, but invert the display every update */
#define LCD_TEST__COLUNMS			((0x3 << 1) | 0x0)  /* Copy an alternating column pattern in the page buffers to show on the LCD (do not show the framebuffer contents) */
#define LCD_TEST__COLUNMS_FLASH			((0x3 << 1) | 0x1)  /* As, LCD_TEST__COLUNMS, but invert the display every update */
#define LCD_TEST__ROWS				((0x4 << 1) | 0x0)  /* Copy an alternating row pattern in the page buffers to show on the LCD (do not show the framebuffer contents) */
#define LCD_TEST__ROWS_FLASH			((0x4 << 1) | 0x1)  /* As, LCD_TEST__ROWS, but invert the display every update */
#define LCD_TEST__IDENTIFY			((0x5 << 1) | 0x0)  /* Copy a pattern spelling 'Identify' in the page buffers to show on the LCD (do not show the framebuffer contents) */
#define LCD_TEST__IDENTIFY_FLASH		((0x5 << 1) | 0x1)  /* As, LCD_TEST__ROWS, but invert the display every update */


enum {
	FPGA_TRACE_NONE = 0,
	FPGA_TRACE_IRQ = (1 << 0),
	FPGA_TRACE_SIZE
};


typedef struct
{
	int           register_base;            /*! identifies i2c bus */
	int           addr;                     /*! destination / source address */
	char          msg[FPGA_I2C_MSG_SIZE];   /*! message to send */
	int           send_len;                 /*! length of sent message */
	int           recv_len;                 /*! length of received message */
} i2c_msg_params;


typedef struct
{
	unsigned int  enable_mask;
	unsigned int  disable_mask;
} irq_params;


#endif /* HBM_FPGA_DRIVER_H */
